@@ -1,34 +1,46 @@
-# Copyright (C) 2009-2011 - Curtis Hovey <sinzui.is at verizon.net>
+# Copyright (C) 2009-2012 - Curtis Hovey <sinzui.is at verizon.net>
 # This software is licensed under the GNU General Public License version 2
 # (see the file COPYING).
 """GDP Gedit Developer Plugins."""
 
-__metaclass__ = type
-
 __all__ = [
+    'config',
     'ControllerMixin',
     'set_file_line',
     'setup_file_lines_view',
     ]
 
 
+from ConfigParser import SafeConfigParser
 import mimetypes
 import os
+import re
+from StringIO import StringIO
 
-from gi.repository import GObject
-from gi.repository import Gio
-from gi.repository import Pango
-from gi.repository import Gtk
+from xdg import BaseDirectory
 
+from gi.repository import (
+    GObject,
+    Gio,
+    Pango,
+    Gtk,
+    GtkSource,
+    )
 
 # Initialise the mimetypes for document type inspection.
 mimetypes.init()
 mimetypes.add_type('application/x-zope-configuation', '.zcml')
 mimetypes.add_type('application/x-zope-page-template', '.pt')
 mimetypes.add_type('text/x-python-doctest', '.doctest')
+mimetypes.add_type('text/x-python', '.tac')
+
+lang_manager = GtkSource.LanguageManager.get_default()
+
+doctest_pattern = re.compile(
+    r'^.*(doc|test|stories).*/.*\.(txt|doctest)$')
 
 
-class ControllerMixin:
+class ControllerMixin(object):
     """Provide common features to plugins"""
 
     def deactivate(self):
@@ -105,6 +117,16 @@ class ControllerMixin:
         end_iter = document.get_end_iter()
         return document.get_text(start_iter, end_iter, True)
 
+    def correct_language(self, document):
+        """Correct the language for ambuguous mime-types."""
+        if not hasattr(document, 'get_language'):
+            return
+        file_path = document.get_uri_for_display()
+        if doctest_pattern.match(file_path):
+            document.set_language(lang_manager.get_language('doctest'))
+        elif file_path.endswith('.tac'):
+            document.set_language(lang_manager.get_language('python'))
+
 
 def set_file_line(column, cell, model, piter, cell_type):
     """Set the value as file or line information."""
@@ -194,3 +216,59 @@ def setup_file_lines_view(file_lines_view, plugin, column_title):
         'row-activated', on_file_lines_row_activated, plugin)
     file_lines_view.connect_after(
         'size-allocate', on_file_lines_resize, column, cell)
+
+
+CONFIG_VERSION = 1
+
+default_conf = """\
+[gdp]
+version = %s
+
+[completer]
+show_accel = <Control>space
+suggest_completions = False
+
+[formatter]
+report_only_errors = False
+
+[finder]
+paths = <Current File>
+    <Working Directory>
+matches =
+files = <Any Text File>
+substitutions =
+""" % CONFIG_VERSION
+
+
+class Config(SafeConfigParser):
+    """A config parser that knows it's defaults and file locations."""
+
+    def __init__(self):
+        SafeConfigParser.__init__(self)
+        self.do_update_state = False
+        self.readfp(StringIO(default_conf), 'default_conf')
+        self.dir_path = os.path.join(BaseDirectory.xdg_config_home, 'gdp')
+        self.file_path = os.path.join(self.dir_path, 'gdp.conf')
+        self._loaded_file_path = None
+
+    def load(self):
+        """Read the optional conf."""
+        self._loaded_file_path = self.read(self.file_path)
+
+    def dump(self):
+        """Write the user changes to the optional conf."""
+        if not os.path.isdir(self.dir_path):
+            os.makedirs(self.dir_path)
+        with open(self.file_path, 'wb') as config_file:
+            self.write(config_file)
+
+    def getlist(self, section, key):
+        return self.get(section, key).split('\n')
+
+    def setlist(self, section, key, value_list):
+        value = '\n'.join(value_list)
+        self.set(section, key, value)
+
+
+config = Config()
+config.load()
